/*! \file apply_list_ddag.hpp
 *  \brief Definitions of the skeleton object ApplyList for DDAG.
 */
#ifndef APPLY_LIST_DDAG_H
#define APPLY_LIST_DDAG_H

#include "../../../util/dpmap_vector.hpp"

namespace skelgis{

  //================================================================================
  //Types of user functions
  //================================================================================
  //! _ApplyListDDAG_Func structure
  /*!
  */
  //-------------------------------------------------------------------------------
  template <class DPMap_Nodes_in, class DPMap_Edges_in, class DPMap_Nodes_out,class DPMap_Edges_out> struct _ApplyListDDAG_Func
  //-------------------------------------------------------------------------------
  {
    virtual void operator()(DPMap_vector<DPMap_Nodes_in>&, DPMap_vector<DPMap_Edges_in>&, DPMap_vector<DPMap_Nodes_out>&, DPMap_vector<DPMap_Edges_out>&) const=0;
  };
  //-------------------------------------------------------------------------------
  template <class DPMap_Nodes, class DPMap_Edges> struct _ApplyListDDAG_Func1
  //-------------------------------------------------------------------------------
  {
    virtual void operator()(DPMap_vector<DPMap_Nodes>&, DPMap_vector<DPMap_Edges>&) const=0;
  };
  //-------------------------------------------------------------------------------
  template <class DPMap_Nodes, class DPMap_Edges, class DPMap_Nodes2,class DPMap_Edges2> struct _ApplyListDDAG_Func2
  //-------------------------------------------------------------------------------
  {
    virtual void operator()(DPMap_vector<DPMap_Nodes>&, DPMap_vector<DPMap_Edges>&, DPMap_vector<DPMap_Nodes2>&, DPMap_vector<DPMap_Edges2>&) const=0;
  };
  //-------------------------------------------------------------------------------

  //================================================================================
  //Specializations of skeleton ApplyList
  //================================================================================
  //! ApplyList structure
  /*!
  */
  //-------------------------------------------------------------------------------
  template<class DPMap_Nodes_in, class DPMap_Edges_in, class DPMap_Nodes_out,class DPMap_Edges_out> void ApplyListDDAG(const _ApplyListDDAG_Func<DPMap_Nodes_in,DPMap_Edges_in,DPMap_Nodes_out,DPMap_Edges_out>& func, 
		      DPMap_vector<DPMap_Nodes_in>& no, DPMap_vector<DPMap_Edges_in>& ed, DPMap_vector<DPMap_Nodes_out>& no2, DPMap_vector<DPMap_Edges_out>& ed2)
  {
    for(unsigned int i=0;i<no.size();i++)
      {
	typename DPMap_Nodes_in::impl_type * im = no.getImpl(i);
	im->start_communications();
	im->end_communications();
      }
    for(unsigned int i=0;i<ed.size();i++)
      {
	typename DPMap_Edges_in::impl_type * im = ed.getImpl(i);
	im->start_communications();
	im->end_communications();
      }
    func(no,ed,no2,ed2);
  }
  //-------------------------------------------------------------------------------
  template<class DPMap_Nodes, class DPMap_Edges> void ApplyListDDAG1(const _ApplyListDDAG_Func1<DPMap_Nodes,DPMap_Edges>& func, DPMap_vector<DPMap_Nodes>& no, DPMap_vector<DPMap_Edges>& ed)
  {
    for(unsigned int i=0;i<no.size();i++)
      {
	typename DPMap_Nodes::impl_type * im = no.getImpl(i);
	im->start_communications();
	im->end_communications();
      }
    for(unsigned int i=0;i<ed.size();i++)
      {
	typename DPMap_Edges::impl_type * im = ed.getImpl(i);
	im->start_communications();
	im->end_communications();
      }
    func(no,ed);
  }
  //-------------------------------------------------------------------------------
  template<class DPMap_Nodes, class DPMap_Edges, class DPMap_Edges2,class DPMap_Edges3> void ApplyListDDAG2(const _ApplyListDDAG_Func2<DPMap_Nodes,DPMap_Edges,DPMap_Edges2,DPMap_Edges3>& func, DPMap_vector<DPMap_Nodes>& no, DPMap_vector<DPMap_Edges>& ed, DPMap_vector<DPMap_Edges2>& ed2, DPMap_vector<DPMap_Edges3>& ed3)
  {
    for(unsigned int i=0;i<no.size();i++)
      {
	typename DPMap_Nodes::impl_type * im = no.getImpl(i);
	im->start_communications();
	im->end_communications();
	}

    for(unsigned int i=0;i<ed.size();i++)
      {
	typename DPMap_Edges::impl_type * im = ed.getImpl(i);
	im->start_communications();
	im->end_communications();
      }

    for(unsigned int i=0;i<ed2.size();i++)
      {
	typename DPMap_Edges2::impl_type * im = ed2.getImpl(i);
	im->start_communications();
	im->end_communications();
      }

    for(unsigned int i=0;i<ed3.size();i++)
      {
	typename DPMap_Edges3::impl_type * im = ed3.getImpl(i);
	im->start_communications();
	im->end_communications();
      }

    func(no,ed,ed2,ed3);
  }
  //-------------------------------------------------------------------------------

  //-------------------------------------------------------------------------------
  template<class DPMap_Nodes, class DPMap_Edges, class DPMap_Edges2,class DPMap_Edges3> void ApplyListDDAG2_wr(const _ApplyListDDAG_Func2<DPMap_Nodes,DPMap_Edges,DPMap_Edges2,DPMap_Edges3>& func, DPMap_vector<DPMap_Nodes>& no, DPMap_vector<DPMap_Edges>& ed, DPMap_vector<DPMap_Edges2>& ed2, DPMap_vector<DPMap_Edges3>& ed3)
  {
    func(no,ed,ed2,ed3);
  }
  //-------------------------------------------------------------------------------

};

#endif
