/*! \file apply_unary_ddag.hpp
 *  \brief Definitions of the skeleton object ApplyUnary. This skeleton takes a user function and a DMatrix as inputs and return a new DMatrix as output.
 */
#ifndef APPLY_UNARY_DDAG_H
#define APPLY_UNARY_DDAG_H

namespace skelgis{

  //================================================================================
  //Types of user functions
  //================================================================================
  //! _ApplyUnaryDDAG_Func structure
  /*!
  */
  //-------------------------------------------------------------------------------
  template <class DPMap_Nodes_in, class DPMap_Edges_in, class DPMap_Nodes_out,class DPMap_Edges_out> struct _ApplyUnaryDDAG_Func
  //-------------------------------------------------------------------------------
  {
    virtual void operator()(DPMap_Nodes_in&, DPMap_Edges_in&, DPMap_Nodes_out&, DPMap_Edges_out&) const =0;
  };

  //================================================================================
  //Specializations of skeleton ApplyUnary
  //================================================================================
  //! ApplyUnary structure
  /*!
  */
  //-------------------------------------------------------------------------------
  template<class DPMap_Nodes_in, class DPMap_Edges_in, class DPMap_Nodes_out,class DPMap_Edges_out> struct ApplyUnaryDDAG
  {
    static void apply(const _ApplyUnaryDDAG_Func<DPMap_Nodes_in,DPMap_Edges_in,DPMap_Nodes_out,DPMap_Edges_out>& func, DPMap_Nodes_in& no, DPMap_Edges_in& ed, DPMap_Nodes_out& no2, DPMap_Edges_out& ed2)
    {
      no.start_communications();
      no.end_communications();
      ed.start_communications();
      ed.end_communications();
      func(no,ed,no2,ed2);
    }
  };
  //-------------------------------------------------------------------------------
}

#endif
