#ifndef MAP_H
#define MAP_H

namespace skelgis{

  //================================================================================
  //Types of user functions for the map skeleton
  //-------------------------------------------------------------------------------
  template <typename T, typename T2> struct _Map_Func
  //-------------------------------------------------------------------------------
  {
    virtual T2 operator()(T element) const =0;
  };
  //================================================================================

  //================================================================================
  //Hidden structure to apply the skeleton map through ApplyUnary
  //-------------------------------------------------------------------------------
  template<class T, int R, class T2, int R2, bool line=true> struct _ApplyUnary_Map : public _ApplyUnary_Func<T, R, T2,R2>
  //-------------------------------------------------------------------------------
  {
    const _Map_Func<T, T2> *func;
    void operator()(DMatrix<T,R>& inp, DMatrix<T2,R2>& outp) const
    {
      iterator<T,R> itBeg = inp.begin();
      iterator<T,R> itEnd = inp.end();
      iterator<T2,R2> itOut = outp.begin();
      for( ; itBeg<itEnd; ++itBeg , ++itOut)
	{
	  outp[itOut] = (*func)(inp[itBeg]);
	}
      }
  };
  //-------------------------------------------------------------------------------
  template<class T, int R, class T2, int R2> struct _ApplyUnary_Map<T,R,T2,R2,false> : public _ApplyUnary_Func<T, R, T2,R2,false>
  //-------------------------------------------------------------------------------
  {
    const _Map_Func<T, T2> *func;
    void operator()(DMatrix<T,R,false>& inp, DMatrix<T2,R2,false>& outp) const
    {
      iterator<T,R> itBeg = inp.begin();
      iterator<T,R> itEnd = inp.end();
      iterator<T2,R2> itOut = outp.begin();
      for( ; itBeg<itEnd; ++itBeg , ++itOut)
	{
	  outp[itOut] = (*func)(inp[itBeg]);
	}
      }
  };
  //================================================================================

  //================================================================================
  //The map skeleton, called by the user
  //no specialization for R=0 because no impact on the map as no exchanges are done
  //-------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------
  template<class T,int R,class T2,int R2,bool line=true> struct Map{};
  //-------------------------------------------------------------------------------

  //-------------------------------------------------------------------------------
  template<class T,int R,class T2,int R2> struct Map<T,R,T2,R2>
  //-------------------------------------------------------------------------------
  {
    //need an operator=(DMatrixw<...> m) in dmatrix.hpp
    /*inline static DMatrix<T2,R2> apply(const _Map_Func<T,T2>& func, DMatrix<T,R>& m)
    {
      DMatrix<T2,R2> m2(m.getGlobalHeader(),0);
      
      _ApplyUnary_Map<T,R,T2,R2> map_f;
      map_f.func = &func;
      ApplyUnary<T,R,T2,R2,true,true>::apply(map_f,m,m2);

      return m2;
      }*/

    inline static void apply(const _Map_Func<T,T2>& func, DMatrix<T,R>& m, DMatrix<T2,R2>& m2)
    {
      _ApplyUnary_Map<T,R,T2,R2> map_f;
      map_f.func = &func;
      ApplyUnary<T,R,T2,R2,true,true>::apply(map_f,m,m2);
    }
  };
  //-------------------------------------------------------------------------------

  //-------------------------------------------------------------------------------
  template<class T,int R,class T2,int R2> struct Map<T,R,T2,R2,false>
  //-------------------------------------------------------------------------------
  {
    //need an operator=(DMatrixw<...> m) in dmatrix.hpp
    /*inline static DMatrix<T2,R2,false> apply(const _Map_Func<T,T2>& func, DMatrix<T,R,false>& m)
    {
      DMatrix<T2,R2,false> m2(m.getGlobalHeader(),0);
      
      _ApplyUnary_Map<T,R,T2,R2,false> map_f;
      map_f.func = &func;
      ApplyUnary<T,R,T2,R2,false,true>::apply(map_f,m,m2);

      return m2;
      }*/

    inline static void apply(const _Map_Func<T,T2>& func, DMatrix<T,R,false>& m, DMatrix<T2,R2,false>& m2)
    {
      _ApplyUnary_Map<T,R,T2,R2,false> map_f;
      map_f.func = &func;
      ApplyUnary<T,R,T2,R2,false,true>::apply(map_f,m,m2);
    }
  };
  //-------------------------------------------------------------------------------
  //================================================================================

}

#endif
