/*! \file dpmap_vector.hpp
 *  \brief Definitions of the object DPMap_vector used by the user when he's manipulating AppyList skeleton on DDAG.
 */
#ifndef DPMAP_VECTOR_H
#define DPMAP_VECTOR_H

#include <vector>

namespace skelgis{

  //================================================================================
  //! DPMap_vector class
  /*!
  */
  //-------------------------------------------------------------------------------
  template<class DPMapType> struct DPMap_vector
  //-------------------------------------------------------------------------------
  {
  protected :
    //! std::vector to store pointers on DPMap_Nodes_impl or DPMap_Edges_impl
    /*!
      This is what is actually stored in the DPMap_vector to hide pointers.
    */
    std::vector<typename DPMapType::impl_type *> data;

  public:
    //-------------------------------------------------------------------------------
    //! default constructor of DMatrix_vector
    //-------------------------------------------------------------------------------
    DPMap_vector(){}
    //-------------------------------------------------------------------------------
    //! destructor of the DMatrix_vector
    //-------------------------------------------------------------------------------
    ~DPMap_vector(){}
    //-------------------------------------------------------------------------------
    //! push_back method of the DPMap_vector class
    /*!
      This method add a new DPMap element at the end of the DPMap_vector. It works as the push_back method of the std::vector object.
      \param dp is the new DPMap to add
    */
    //-------------------------------------------------------------------------------
    inline void push_back(DPMapType& dp){data.push_back(dp.getDPMap());}
    //-------------------------------------------------------------------------------
    //! operator [] of DPMap_vector
    /*!
      Similar to table brackets or to std::vector brackets to access an element of the DPMap_vector
      \param n is the index element to access in the DPMap_vector
      \return Returns a pointer to the element at position n in the DPMap_vector.
    */
    //-------------------------------------------------------------------------------
    inline DPMapType operator[](int n){return DPMapType(data[n]);}
    //-------------------------------------------------------------------------------
    //! get impl of dpmap of DPMap_vector
    /*!                                                                                                                   
      directly get what is in the DPMap_vector
      \param i is the index to get
      \return Return the DPMap_.._impl
    */
    //-------------------------------------------------------------------------------
    inline typename DPMapType::impl_type* getImpl(unsigned int i){return data[i];}
    //-------------------------------------------------------------------------------
    //! size method of the DPMap_vector class
    /*!
      This method returns the size of the DPMap_vector, in other words the number of elements.
    */
    //-------------------------------------------------------------------------------
    inline int size(){return data.size();}
    //-------------------------------------------------------------------------------
    //! clear method of the DPMap_vector class
    /*!
      Removes all elements from the DPMap_vector, leaving the container with a size of 0
    */
    //-------------------------------------------------------------------------------
    inline void clear(){data.clear();}
    //-------------------------------------------------------------------------------
  };
}

#endif
